from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
from mygnuhealth.fedlogin import test_federation_connection as fc
import datetime
import bcrypt

class NetworkSettings(QObject):
    def __init__(self):
        QObject.__init__(self)

    db = TinyDB(dbfile)

    def update_federation_info(self, protocol, federation_server, federation_port,
                               federation_id, enable_sync):

        fedinfo = self.db.table('federation')
        #If the "Singleton" table is empty, insert, otherwise, update
        #TODO: Use upsert with doc_id == 1 as condition
        if (len(fedinfo) == 0):
            fedinfo.insert({'protocol':protocol,
                            'federation_server':federation_server,
                            'federation_port':federation_port,
                            'federation_id':federation_id,
                            'enable_sync':enable_sync})
        else:
            fedinfo.update({'protocol':protocol,
                            'federation_server':federation_server,
                            'federation_port':federation_port,
                            'federation_id':federation_id,
                            'enable_sync':enable_sync})
            

    @Slot (str,str,str,str,str)
    def test_connection(self,protocol, federation_server, federation_port,
                        federation_id, password):
        conn_res = fc(protocol, federation_server, federation_port, federation_id,
                      password)

        print (conn_res)


    @Slot (str,str,str, str,bool)
    def getvals(self,protocol, federation_server, federation_port, federation_id,
                enable_sync):
        self.update_federation_info(protocol, federation_server, federation_port,
                               federation_id, enable_sync)
        #self.setOK.emit()


    # Signal to emit to QML if the values were stored correctly
    setOK = Signal()



from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
import datetime

class Osat(QObject):
    def __init__(self):
        QObject.__init__(self)

    db = TinyDB(dbfile)

    def insert_values(self, hb_osat):
        osat = self.db.table('osat')
        current_date = datetime.datetime.now().isoformat()

        if ((hb_osat > 0)):
            osat.insert({'timestamp': current_date,
                                   'osat': hb_osat})

            print ("Saved osat",hb_osat, current_date)


    @Slot (int)
    def getvals(self,hb_osat):
        self.insert_values(hb_osat)
        self.setOK.emit()

    # Signal to emit to QML if the body pressure values were stored correctly
    setOK = Signal()



from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import get_personal_key
import datetime
import bcrypt

class ProfileSettings(QObject):
    def __init__(self):
        QObject.__init__(self)

    db = TinyDB(dbfile)

    def check_current_password(self, current_password):
        personal_key = get_personal_key(self.db)
        cpw = current_password.encode()
        if (bcrypt.checkpw(cpw, personal_key)):
            rc = True
        else:
            print("Wrong current password")
            rc = False
        return rc

    def check_new_password(self, password, password_repeat):
        if (password == password_repeat):
            rc = True
        else:
            print("new passwords do not match")
            rc = False
        return rc

    def update_personalkey(self, password):
        encrypted_key = bcrypt.hashpw(password.encode('utf-8'), \
            bcrypt.gensalt()).decode('utf-8')

        credentials = self.db.table('credentials')
        credentials.update({'personal_key':encrypted_key})

        print ("Saved personal key", encrypted_key)


    @Slot (str, str, str)
    def getvals(self,current_password, password, password_repeat):
        if (self.check_current_password(current_password) and
            self.check_new_password(password, password_repeat)):
            self.update_personalkey(password)
            self.setOK.emit()

    # Signal to emit to QML if the password was stored correctly
    setOK = Signal()



#!/usr/bin/env python3

##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2020 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2020 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import os
import sys
import bcrypt
import getpass
import tinydb
from pathlib import Path
from tinydb import TinyDB, Query
import configparser

homedir = str(Path.home())
gh_dir = os.path.join(homedir, 'mygh')
config_file = os.path.join(gh_dir, 'ghealth.conf')
dbfile = os.path.join(gh_dir, 'ghealth.db')
GH_KEY = "mygh"

def check_inst_dir():
    if (os.path.isdir(gh_dir)):
        print ("Directory exists... skipping\n")
    else:
        print ("Initializing MyGNUHealth directory")
        try:
            os.mkdir(gh_dir)
        except:
            print ("Error initializing MyGNUHealth directory")

def check_config():
    if os.path.isfile(config_file):
        print ("Found myGNUHealth configuration file.. skipping")
    else:
        print ("Configuration file not found. Writing defaults")
        set_default_config_file()

def check_db():
    print ("Verifying MyGNUHealth Database.....")
    if os.path.isfile(dbfile):
        print ("MyGNUHealth DB exists.. skipping")
    else:
        print ("DB file not found. Initializing MyGNUHealth...")
        db = TinyDB(dbfile)
        init_db(db)

def set_default_config_file():
    config = configparser.ConfigParser()
    config.read(config_file)
    if not 'security' in config.sections():
        config.add_section('security')

    config.set ('security','key_method','bcrypt')
    output_file = open(config_file,'w')
    config.write(output_file)

def init_db(db):
    encrypted_key = bcrypt.hashpw(GH_KEY.encode('utf-8'), \
        bcrypt.gensalt()).decode('utf-8')

    credentials = db.table('credentials')
    credentials.insert({'personal_key':encrypted_key})

    print ("MyGNUHealth DB initialized !")


def verify_installation_status():
    print ("Initializing myGNUHealth PHR....")
    check_inst_dir()
    check_config()
    check_db()
    return True



import requests


def test_federation_connection(protocol,host, port, acct, passwd):
    """ Make the connection test to Thalamus Server
        from the GNU Health HMIS using the institution
        associated admin and the related credentials
    """
    conn = ''

    url = protocol +"://" + host + ':' + str(port) + '/people/' + acct

    try:
        conn = requests.get(url,
            auth=(acct, passwd), verify=False)

    except:
        print ("ERROR authenticating to Server")
        login_status = -2


    if conn:
        print ("***** Connection to Thalamus Server OK !******")
        login_status = 0

    else:
        print ("##### Wrong credentials ####")
        login_status = -1

    print (login_status)
    return (login_status)




from PySide2.QtCore import QObject, Signal, Slot, Property
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
import datetime
import matplotlib.pyplot as plt
import io
import base64
from mygnuhealth.core import datefromisotz

class GHBio(QObject):
    def __init__(self):
        QObject.__init__(self)

        self.current_bp = ""
        self.current_glucose = ""
        self.current_weight = ""
        self.current_osat = ""

    db = TinyDB(dbfile)

    def read_bp(self):
        #Retrieve the BP history
        blood_pressure = self.db.table('bloodpressure')
        bphist = blood_pressure.all()
        return (bphist)

    def read_hr(self):
        #Retrieve the HR history
        hr = self.db.table('heart_rate')
        hrhist = hr.all()
        return (hrhist)

    def getBP(self):
        # Extracts the latest readings from BP
        bphist = self.read_bp()
        hrhist = self.read_hr()
        bp = bphist[-1]  # Get the latest (newest) record
        hr = hrhist[-1]

        #dateobj =  datetime.datetime.fromisoformat(bp['timestamp'])
        dateobj =  datefromisotz(bp['timestamp'])
        date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

        bpobj = [str(date_repr), str(bp['systolic']), str(bp['diastolic']),
                                     str(hr['heart_rate'])]
        return bpobj

    def getBPHist(self):
        # Retrieves all the history and packages into an array.
        bphist = self.read_bp()
        hrhist = self.read_hr()
        bphrhist = []

        bpsys = []
        bpdia = []
        hr = []
        for element in bphist:
            bpsys.append(element['systolic'])
            bpdia.append(element['diastolic'])
        bphrhist.append(bpsys)
        bphrhist.append(bpdia)

        for element in hrhist:
            hr.append(element['heart_rate'])
        bphrhist.append(hr)

        return bphrhist


    def BPplot(self):
        # Retrieves all the history and packages into an array.
        bphist = self.read_bp()
        bpsys = []
        bpdia = []
        bp_date = []
        lastreading=''
        for element in bphist:
            #dateobj =  datetime.datetime.fromisoformat(element['timestamp'])
            dateobj =  datefromisotz(element['timestamp'])

            date_repr = dateobj.strftime("%a, %b %d '%y")

            # Only print one value per day to avoid artifacts in plotting.
            if (lastreading != date_repr):
                bp_date.append(dateobj)
                bpsys.append(element['systolic'])
                bpdia.append(element['diastolic'])

            lastreading = date_repr

        fig, axs = plt.subplots(2)

        #Plot both systolic and diastolic history
        axs[0].plot(bp_date, bpsys)
        axs[1].plot(bp_date, bpdia, color='teal')

        axs[0].set_ylabel('Systolic', size=13)
        axs[1].set_ylabel('Diastolic', size=13)

        fig.autofmt_xdate()
        fig.suptitle("Blood Pressure (mm Hg)",size=20)
        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)


    def HRplot(self):
        # Retrieves all the history and packages into an array.
        hrhist = self.read_hr()
        hr = []
        hr_date= []
        lastreading=''
        for element in hrhist:
            #dateobj =  datetime.datetime.fromisoformat(element['timestamp'])
            dateobj =  datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            if (lastreading != date_repr):
                hr_date.append(dateobj)
                hr.append(element['heart_rate'])

            lastreading = date_repr

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(hr_date, hr, color="orange")

        ax.set_ylabel('Frequency',size=13)
        fig.autofmt_xdate()
        fig.suptitle("Heart Rate (bpm)",size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)


    def setBP(self, bp):
        self.current_bp = bp
        # Call the notifying signal
        self.bpChanged.emit()



    # GLUCOSE
    def setGlucose(self, glucose):
        self.current_glucose = glucose
        # Call the notifying signal
        self.glucoseChanged.emit()

    def read_glucose(self):
        #Retrieve the blood glucose levels history
        glucose = self.db.table('glucose')
        glucosehist = glucose.all()
        return (glucosehist)


    def getGlucose(self):
        # Extracts the latest readings from Glucose
        glucosehist = self.read_glucose()
        glucose = glucosehist[-1]  # Get the latest (newest) record

        #dateobj =  datetime.datetime.fromisoformat(glucose['timestamp'])
        dateobj =  datefromisotz(glucose['timestamp'])
        date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

        glucoseobj = [str(date_repr), str(glucose['glucose'])]
        return glucoseobj

    def getGlucoseHist(self):
        # Retrieves all the history and packages into an array.
        glucosehist = self.read_glucose()

        glucose = []
        for element in glucosehist:
            glucose.append(element['glucose'])
        return glucose

    def Glucoseplot(self):
        # Retrieves all the history and packages into an array.
        glucosehist = self.read_glucose()
        glucose = []
        glucose_date= []
        lastreading=''
        for element in glucosehist:
            #dateobj =  datetime.datetime.fromisoformat(element['timestamp'])
            dateobj =  datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            #if (lastreading != date_repr):
            glucose_date.append(dateobj)
            glucose.append(element['glucose'])

            #lastreading = date_repr

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(glucose_date, glucose, color="red")

        ax.set_ylabel('mg/dl',size=13)
        fig.autofmt_xdate()
        fig.suptitle("Glucose level (mg/dl)",size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)

    # WEIGHT
    def setWeight(self, weight):
        self.current_weight = weight
        # Call the notifying signal
        self.weightChanged.emit()

    def read_weight(self):
        #Retrieve the blood weight levels history
        weight = self.db.table('weight')
        weighthist = weight.all()
        return (weighthist)


    def getWeight(self):
        # Extracts the latest readings from Weight
        weighthist = self.read_weight()
        weight = weighthist[-1]  # Get the latest (newest) record

        #dateobj =  datetime.datetime.fromisoformat(weight['timestamp'])
        dateobj =  datefromisotz(weight['timestamp'])
        date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

        weightobj = [str(date_repr), str(weight['weight'])]
        return weightobj

    def getWeightHist(self):
        # Retrieves all the history and packages into an array.
        weighthist = self.read_weight()

        weight = []
        for element in weighthist:
            weight.append(element['weight'])
        return weight

    def Weightplot(self):
        # Retrieves all the history and packages into an array.
        weighthist = self.read_weight()
        weight = []
        weight_date= []
        lastreading=''
        for element in weighthist:
            #dateobj =  datetime.datetime.fromisoformat(element['timestamp'])
            dateobj =  datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            #if (lastreading != date_repr):
            weight_date.append(dateobj)
            weight.append(element['weight'])

            #lastreading = date_repr

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(weight_date, weight, color="blue")

        ax.set_ylabel('kg',size=13)
        fig.autofmt_xdate()
        fig.suptitle("Weight (kg)",size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)



    # OSAT
    def setOsat(self, osat):
        self.current_osat = osat
        # Call the notifying signal
        self.osatChanged.emit()

    def read_osat(self):
        #Retrieve the blood osat levels history
        osat = self.db.table('osat')
        osathist = osat.all()
        return (osathist)


    def getOsat(self):
        # Extracts the latest readings from Osat
        osathist = self.read_osat()
        osat = osathist[-1]  # Get the latest (newest) record

        #dateobj =  datetime.datetime.fromisoformat(osat['timestamp'])
        dateobj =  datefromisotz(osat['timestamp'])
        date_repr = dateobj.strftime("%a, %b %d '%y - %H:%M")

        osatobj = [str(date_repr), str(osat['osat'])]
        return osatobj

    def getOsatHist(self):
        # Retrieves all the history and packages into an array.
        osathist = self.read_osat()

        osat = []
        for element in osathist:
            osat.append(element['osat'])
        return osat

    def Osatplot(self):
        # Retrieves all the history and packages into an array.
        osathist = self.read_osat()
        osat = []
        osat_date= []
        lastreading=''
        for element in osathist:
            #dateobj =  datetime.datetime.fromisoformat(element['timestamp'])
            dateobj =  datefromisotz(element['timestamp'])
            date_repr = dateobj.strftime("%a, %b %d '%y")
            # Only print one value per day to avoid artifacts in plotting.
            #if (lastreading != date_repr):
            osat_date.append(dateobj)
            osat.append(element['osat'])

            #lastreading = date_repr

        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

        ax.plot(osat_date, osat, color="red")

        ax.set_ylabel('%',size=13)
        fig.autofmt_xdate()
        fig.suptitle("Osat (%)",size=20)

        holder = io.BytesIO()
        fig.savefig(holder, format="svg")
        image = "data:image/svg+xml;base64," + \
            base64.b64encode(holder.getvalue()).decode()

        holder.close()
        return (image)


    # PROPERTIES BLOCK

    # Notifying signal - to be used in qml as "onBPChanged"
    bpChanged = Signal()

    # BP property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    bp = Property("QVariantList", getBP, setBP, notify=bpChanged)

    # Property to retrieve the plot of the Blood pressure.
    bpplot = Property(str, BPplot, setBP, notify=bpChanged)

    # Retrieve the heart rate history.
    # I made a different plot because the dates can differ from those of the 
    # Blood pressure monitor readings.
    hrplot = Property(str, HRplot, setBP, notify=bpChanged)


    # Notifying signal - to be used in qml as "onGlucoseChanged"
    glucoseChanged = Signal()

    # Glucose property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    glucose = Property("QVariantList", getGlucose, setGlucose, notify=glucoseChanged)

    # Property to retrieve the plot of the blood glucose level.
    glucoseplot = Property(str, Glucoseplot, setGlucose, notify=bpChanged)


    # Notifying signal - to be used in qml as "onWeightChanged"
    weightChanged = Signal()

    # Weight property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    weight = Property("QVariantList", getWeight, setWeight, notify=weightChanged)

    # Property to retrieve the plot of the blood weight level.
    weightplot = Property(str, Weightplot, setWeight, notify=bpChanged)


    # Notifying signal - to be used in qml as "onOsatChanged"
    osatChanged = Signal()

    # Osat property to be accessed to and from QML and Python.
    # It is used in the context of showing the BP last results
    # in the main bio screen.
    osat = Property("QVariantList", getOsat, setOsat, notify=osatChanged)

    # Property to retrieve the plot of the blood osat level.
    osatplot = Property(str, Osatplot, setOsat, notify=bpChanged)



from PySide2.QtCore import QObject, Signal, Slot, Property
import bcrypt
from tinydb import TinyDB, Query
from mygnuhealth.myghconf import dbfile
from mygnuhealth.core import get_personal_key

class GHLogin(QObject):
    def __init__(self):
        QObject.__init__(self)

    db = TinyDB(dbfile)

    '''
    def get_personal_key(self, db):
        credentials = db.table('credentials')
        # Credentials table holds a singleton, so only one record
        personal_key = credentials.all()[0]['personal_key']
        return personal_key.encode()
    '''

    @Slot (str)
    def getKey(self,key):

        key = key.encode()

        # personal_key = self.get_personal_key(self.db)
        personal_key = get_personal_key(self.db)

        if (bcrypt.checkpw(key, personal_key)):
            print ("Login correct - Move to main PHR page")
            self.loginOK.emit()

    # Signal to emit to QML if the provided credentials are correct
    loginOK = Signal()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
##############################################################################
#
#    MyGNUHealth : Mobile and Desktop PHR node for GNU Health
#
#           MyGNUHealth is part of the GNU Health project
#
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2020 Luis Falcon <falcon@gnuhealth.org>
#    Copyright (C) 2011-2020 GNU Solidario <health@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import sys
import os
from PySide2.QtWidgets import QApplication
from PySide2.QtCore import QObject, QUrl, Signal, Slot
from PySide2.QtQml import QQmlApplicationEngine, qmlRegisterType

from mygnuhealth.myghconf import verify_installation_status

import dateutil.parser


#Common methods
#Use this method to be compatible with Python 3.6
# datetime fromisoformat is not present until Python 3.7
def datefromisotz (isotz):
    if isotz:
        return (dateutil.parser.parse(isotz))

def main():
    
    # Initial installation check
    if (verify_installation_status()):
        from mygnuhealth.profile_settings import ProfileSettings
        from mygnuhealth.network_settings import NetworkSettings
        from mygnuhealth.ghlogin import GHLogin
        from mygnuhealth.bio import GHBio

        from mygnuhealth.bloodpressure import BloodPressure
        from mygnuhealth.glucose import Glucose
        from mygnuhealth.weight import Weight
        from mygnuhealth.osat import Osat

    app = QApplication(sys.argv)

    # Register ProfileSettings to use in QML
    qmlRegisterType(ProfileSettings, "ProfileSettings", 0, 1,
                    "ProfileSettings")

    # Register NetworkSettings to use in QML
    qmlRegisterType(NetworkSettings, "NetworkSettings", 0, 1,
                    "NetworkSettings")


    # Register BloodPressure to use in QML
    qmlRegisterType(BloodPressure, "BloodPressure", 0, 1,
                    "BloodPressure")

    # Register Glucose to use in QML
    qmlRegisterType(Glucose, "Glucose", 0, 1,
                    "Glucose")

    # Register Weight to use in QML
    qmlRegisterType(Weight, "Weight", 0, 1,
                    "Weight")

    # Register Osat to use in QML
    qmlRegisterType(Osat, "Osat", 0, 1,
                    "Osat")

    # Register GHLogin to use in QML
    qmlRegisterType(GHLogin, "GHLogin", 0, 1,
                    "GHLogin")

    # Register GHBio to use in QML
    qmlRegisterType(GHBio, "GHBio", 0, 1,
                    "GHBio")

    engine = QQmlApplicationEngine()

    basePath=os.path.abspath(os.path.dirname(__file__))
    url = QUrl('file://'+basePath+'/qml/main.qml')
    engine.load(url)

    if not engine.rootObjects():
        sys.exit(-1)

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()
